﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.ViewModels;
using AbstractIceCreamShopDataModels.Models;
using AbstractIceCreamShopDataModels.Enums;
using System.Data;
using System.Reflection;
using System.Runtime.Serialization;

namespace IceCreamShopListImplement.Models 
{
    [DataContract]
	public class Order : IOrderModel
    {
        [DataMember]
		public int Id { get; private set; }

		[DataMember]
		public int IceCreamId { get; private set; }

        [DataMember]
		public int ClientId { get; private set; }

        [DataMember]
		public int? ImplementerId { get; private set; }

        [DataMember]
		public int Count { get; private set; }

        [DataMember]
		public double Sum { get; private set; }

        [DataMember]
		public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        [DataMember]
		public DateTime DateCreate { get; private set; } = DateTime.Now;

        [DataMember]
		public DateTime? DateImplement { get; private set; }
        
        public static Order? Create (OrderBindingModel? model)
        {
            if (model == null) return null;
            return new Order
            {
                Id = model.Id,
                IceCreamId = model.IceCreamId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }    
        
        public void Update(OrderBindingModel? model)
        {
            if (model == null) return;
            Status = model.Status;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            IceCreamId = IceCreamId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
